#include "main.h"
#include "gui.h"
#include "chips.h"

extern int tf;

/* Destroy the main window. */
gint destroy_gui( GtkWidget *widget, gpointer data ) {

    gtk_main_quit();

    return (FALSE);
}

/* Free the link list. */
int free_llist( updates *node ) {

    if ( node->next == NULL ) {
        free( node );
    } else {
        free_llist( node->next );
        node->next = NULL;
        free_llist( node );
    }

    return SUCCESS;
}

/* Update the sensor information. */
gint update_sensor_data( gpointer data ) {
    double featureresult = 0;
    double over = 0;
    double under = 0;
    char *afeatresult;
    updates *updata = data;

    do {
        if ( ( afeatresult = malloc( sizeof(char) * 15 ) ) == NULL ) {
            fprintf( stderr, "malloc failed!\n" );
        }

        if ( sensors_get_feature( updata->name, updata->featnum,
             &featureresult ) != 0 ) {
                featureresult = 0;
        }
        
        if ( updata->featminnum != UNDEFMAXMIN ) {
            sensors_get_feature( updata->name, updata->featminnum, &under );
        } else {
            under = -100000;
        }
        if ( updata->featmaxnum != UNDEFMAXMIN ) {
            sensors_get_feature( updata->name, updata->featmaxnum, &over );
        } else {
            over = 100000;
        }

        switch ( updata->feattype ) {
            case FAN:
                sprintf( afeatresult, "%6.0f", featureresult );
                afeatresult = strcat( afeatresult, " RPM" );
                break;
            case TEMP:
                if ( tf == TRUE ) {
                    featureresult = 1.8 * featureresult + 32;
                    sprintf( afeatresult, "%6.1f", featureresult );
                    afeatresult = strcat( afeatresult, " F" );
                } else {
                    sprintf( afeatresult, "%6.1f", featureresult );
                    afeatresult = strcat( afeatresult, " C" );
                }
                break;
            case VOLT:
                sprintf( afeatresult, "%6.2f", featureresult );
                afeatresult = strcat( afeatresult, " V" );
                break;
            case EEPROM:
                sprintf( afeatresult, "%6.0f", featureresult );
                break;
            default:
                sprintf( afeatresult, "%6.2f", featureresult );
                break;
        }

        if ( ( featureresult < under ) || ( featureresult > over ) ) {
        }
        gtk_entry_set_text( GTK_ENTRY (updata->widget), afeatresult );

        updata = updata->next;

        free( afeatresult );
    } while ( updata != NULL );

    return (TRUE);
}

/* Start the sensor info update timer. */
gint start_timer( GtkWidget *widget, gpointer data ) {
    gint timer;

    /* Setup timer for updates. */
    timer = gtk_timeout_add( 1000, (GtkFunction) update_sensor_data, 
            (gpointer) data );

    return SUCCESS;
}

/* Switch the sensor that is being updated. */
/* CURRENTLY NOT IMPLEMENTED! */
gint switch_sensor( GtkWidget *widget, GtkNotebook *notebook,
        GtkNotebookPage *page, gint page_num, gpointer data ) {
#ifdef DEBUG
    g_print( "Switched sensor.\n" );
#endif

    return SUCCESS;
}

updates *add_sensor_tab( GtkWidget *container, sensors_chip_name name ) {
    /* packing boxes */
    GtkWidget *mainbox = NULL;
    GtkWidget *voltbox = NULL;
    GtkWidget *tempbox = NULL;
    GtkWidget *fanbox = NULL;
    GtkWidget *eeprombox = NULL;
    GtkWidget *currbox = NULL;
   
    /* notebook stuff */
    GtkWidget *notelabel = NULL;
    GtkWidget *noteframe = NULL;

    /* main feature labels */
    GtkWidget *voltlabel = NULL;
    GtkWidget *templabel = NULL;
    GtkWidget *fanlabel = NULL;
    GtkWidget *eepromlabel = NULL;

    /* feature data */
    updates *head = NULL;
    updates *current = NULL;

    /* several needed ints */
    int numfeat = 0;
    int i = 0;
    int usedvolt = 0;
    int usedtemp = 0;
    int usedfan = 0;
    int usedeeprom = 0;
   
    /* fields placed in the notebook to display feature info */
    GtkWidget *featlabel = NULL;
    char *feattext = NULL;
    GtkWidget *featentry = NULL;
 
    /* Setup main boxes. */
    mainbox = gtk_hbox_new( TRUE, 10 );
    gtk_container_border_width( GTK_CONTAINER (mainbox), 10 );
    gtk_widget_show( mainbox );
    voltbox = gtk_vbox_new( FALSE, 0 );
    tempbox = gtk_vbox_new( FALSE, 0 );
    fanbox = gtk_vbox_new( FALSE, 0 );
    eeprombox = gtk_vbox_new( FALSE, 0 );

    /* Create notebook for sensors. */
    noteframe = gtk_frame_new( NULL );
    gtk_container_set_border_width( GTK_CONTAINER (noteframe), 10 );
    gtk_widget_show( noteframe );
    
    notelabel = gtk_label_new( name.prefix );
    gtk_widget_show( notelabel );

    gtk_container_add( GTK_CONTAINER (noteframe), mainbox );
    gtk_notebook_append_page( GTK_NOTEBOOK (container), noteframe, notelabel );
    
    /* Create main labels. */
    voltlabel = gtk_label_new( "Voltages:" );
    templabel = gtk_label_new( "Temperatures:" );
    fanlabel = gtk_label_new( "Fans:" );
    eepromlabel = gtk_label_new( "EEPROM:" );
    gtk_box_pack_start( GTK_BOX (voltbox), voltlabel, FALSE, FALSE, 0 );
    gtk_box_pack_start( GTK_BOX (tempbox), templabel, FALSE, FALSE, 0 );
    gtk_box_pack_start( GTK_BOX (fanbox), fanlabel, FALSE, FALSE, 0 );
    gtk_box_pack_start( GTK_BOX (eeprombox), eepromlabel, FALSE, FALSE, 0 );

    /* Create labels and entry fields for features. */
    numfeat = get_num_feat( name );
    for ( i = 1; i <= numfeat; i++ ) {
        if ( sensors_get_ignored( name, i ) != 0 ) {
            featentry = gtk_entry_new();
            if ( head == NULL ) {
                head = current = add_node( head, name, featentry, i );
            } else {
                current = current->next = 
                    add_node( current->next, name, featentry, i );
            }
            sensors_get_label( name, current->featnum, &feattext );
            if ( feattext != NULL ) {
#ifdef DEBUG
                printf( "Adding feature %d, %s.\n", i, feattext );
#endif
                if ( ( feattext = realloc( feattext, 
                                ( strlen( feattext ) + 2 ) * 
                                sizeof( char ) ) ) == NULL ) {
                    fprintf( stderr, "realloc failed in add_sensor_tab()!\n" );
                    return NULL;
                }
                if ( strcat( feattext, ":" ) == NULL ) {
                    fprintf( stderr, "strcat failed in add_sensor_tab()!\n" );
                    return NULL;
                }
                featlabel = gtk_label_new( feattext );
                switch ( current->feattype ) {
                    case VOLT:
                        currbox = voltbox;
                        usedvolt++;
                        break;
                    case TEMP:
                        currbox = tempbox;
                        usedtemp++;
                        break;
                    case FAN:
                        currbox = fanbox;
                        usedfan++;
                        break;
                    case EEPROM:
                        currbox = eeprombox;
                        usedeeprom++;
                        break;
                    default:
                        fprintf( stderr, 
                                 "Type not recognized, not packing.\n" );
                        break;
                }
                gtk_box_pack_start( GTK_BOX (currbox), featlabel, 
                                    FALSE, FALSE, 0 );
                gtk_box_pack_start( GTK_BOX (currbox), featentry, 
                                    FALSE, FALSE, 0 );
                gtk_entry_set_editable( GTK_ENTRY (featentry), FALSE );
            } else {
                featlabel = gtk_label_new( NA );
            }
            gtk_widget_show( featlabel ); gtk_widget_show( featentry );
            free( feattext );
            feattext = NULL;
        }
#ifdef DEBUG
        else {
            printf( "Feature %d is ignored.\n", i );
        }
#endif
    }

    if ( usedvolt > 0 ) {
        gtk_widget_show( voltbox );
        gtk_box_pack_start( GTK_BOX (mainbox), voltbox, FALSE, FALSE, 0 );
        gtk_widget_show( voltlabel );
    }

    if ( usedtemp > 0 ) {
        gtk_widget_show( tempbox );
        gtk_box_pack_start( GTK_BOX (mainbox), tempbox, FALSE, FALSE, 0 );
        gtk_widget_show( templabel );
    }

    if ( usedfan > 0 ) {
        gtk_widget_show( fanbox );
        gtk_box_pack_start( GTK_BOX (mainbox), fanbox, FALSE, FALSE, 0 );
        gtk_widget_show( fanlabel );
    }

    if ( usedeeprom > 0 ) {
        gtk_widget_show( eeprombox );
        gtk_box_pack_start( GTK_BOX (mainbox), eeprombox, FALSE, FALSE, 0 );
        gtk_widget_show( eepromlabel );
    }

    free( feattext );
    free( feattext );

    return head;
}

int start_gui( int argc, char **argv ) {
    GtkWidget *mainwindow = NULL;
    GtkWidget *notebook = NULL;
    
    updates *head = NULL;

    int chipnum = 0;
    sensors_chip_name *name = NULL;

    gtk_init( &argc, &argv );

    /* Setup main window. */
    mainwindow = gtk_window_new( GTK_WINDOW_TOPLEVEL );
    gtk_window_set_title( GTK_WINDOW (mainwindow), "xsensors" );
    gtk_signal_connect( GTK_OBJECT (mainwindow), "delete_event",
            GTK_SIGNAL_FUNC (destroy_gui), NULL );
    
    /* Create notebook for sensors. */
    notebook = gtk_notebook_new( );
    gtk_notebook_set_tab_pos( GTK_NOTEBOOK (notebook), GTK_POS_LEFT );
    gtk_widget_show( notebook );

    gtk_container_add( GTK_CONTAINER (mainwindow), notebook );

    while ( ( name = sensors_get_detected_chips( &chipnum ) ) != NULL ) {
        if ( get_num_feat( *name ) > 0 ) {
#ifdef DEBUG
            printf( "Adding tab for %s\n", name->prefix );
#endif
            head = add_sensor_tab( notebook, *name );
            update_sensor_data( head );
            gtk_signal_connect( GTK_OBJECT (mainwindow), "realize",
                    GTK_SIGNAL_FUNC (start_timer), head );
        } else {
            fprintf( stderr, "Sensor '%s' not supported by xsensors!\n", 
                     name->prefix );
        }
    }
    
    /* Setup the main components. */
    gtk_signal_connect( GTK_OBJECT (notebook), "switch-page",
            GTK_SIGNAL_FUNC (switch_sensor), NULL );
    gtk_widget_show( mainwindow );

    gtk_main();

    free_llist( head );
    
    return SUCCESS;
}
