#include "main.h"
#include "gui.h"

/* getopt stuff */
extern int      optopt;
extern int      optind;
extern char    *optarg;
extern int      opterr;

int             getopt (int, char *const *, const char *);
/* end getopt stuff */

char *strdup(const char *s);

/* Our public vars */
int tf = FALSE;

/* Print the help message. */
int help_msg( void ) {
    printf( "Usage: xsensors [options]\n\n"
            "Options:\n"
            "--------\n\n"
            "-f\t\tDisplay all temperatures in Fahrenheit.\n"
            "-h\t\tDisplay this help text and exit.\n"
            "-c filename\tSpecify the lm_sensors configuration file.\n"
            "\n" );

    return SUCCESS;
}

/* Prints out the chips' labels and features. Mainly used for debugging. */
int print_features( sensors_chip_name name ) {
    int featnum = 1;
    char *labelresult = NULL;
    double featureresult = 0;

    printf( "Sensor: %s\n", name.prefix );

    /* Aparently none of the features are defined over 99 so go through all
     * up until that and list those that return successfully.
     */
    while ( featnum < 99 ) {

        if ( sensors_get_label( name, featnum, &labelresult ) == 0 ) {
            if ( sensors_get_feature( name, featnum, &featureresult ) != 0 ) {
                fprintf( stderr, "Could not get feature!\n" );
                return FAILURE;
            }
        }
       
        if ( labelresult != NULL ) {
            printf( "%s: %f\n", labelresult, featureresult );
        }

        featnum++;
    }

    free( labelresult );

    return SUCCESS;
}

/* Main. */
int main( int argc, char **argv ) {
    int c = 0;
    char *sens_config = NULL;
    FILE *sens_conf_file = NULL;

    /* Process arguements. */
    while ( ( c = getopt( argc, argv, "fhc:" ) ) != EOF ) {
        switch (c) {
            case 'f':
                tf = TRUE;
                break;
            case 'h':
                if ( help_msg() == SUCCESS )
                    return EXIT_SUCCESS;
                else
                    return EXIT_FAILURE;
            case 'c':
                sens_config = optarg;
                break;
            case '?':
                return EXIT_FAILURE;
            default:
                fprintf( stderr, "Something has gone wrong!\n" );
                return EXIT_FAILURE;
        }
    }

    /* If lm_sensors config file was not specified, use default. */
    if ( sens_config == NULL )
        sens_config = strdup( DEFAULT_SENS_CONF );

    /* Open the config file. */    
    if ( ( sens_conf_file = fopen( sens_config, "r" ) ) == NULL ) {
        fprintf( stderr, "Error opening config file: %s\n"
                 "Use -c option to specify location of lm_sensors"
                 " configuration file.\n"
                 , sens_config );
        return EXIT_FAILURE;
    }

    free( sens_config );

    /* Initialize the sensors library. */
    if ( sensors_init( sens_conf_file ) != SUCCESS ) {
        fprintf( stderr, "Could not initialize sensors!\n"
                 "Is everything installed properly?\n" );
        return EXIT_FAILURE;
    }

    /* This will start the GUI. */
    if ( start_gui( argc, argv ) != SUCCESS ) {
        fprintf( stderr, "GUI failed!\n" );
    }

    /* Clean up the sensors library. */
    sensors_cleanup();
    
    /* Close the config file. */
    if ( fclose( sens_conf_file ) != SUCCESS ) {
        fprintf( stderr, 
                 "Something has gone wrong closing the config file!\n" );
        return EXIT_FAILURE;
    }

    return EXIT_SUCCESS;
}
